#include <math.h>
#include "Day.h"

int Day::get_year()
{
    ensure_ymd();
    return year;
}

int Day::get_month()
{
    ensure_ymd();
    return month;
}

int Day::get_date()
{
    ensure_ymd();
    return date;
}

Day *Day::add_days(const int n)
{
    ensure_julian();
    return new Day(julian + n);
}

int Day::days_from(Day *other)
{
    ensure_julian();
    other->ensure_julian();
    return julian - other->julian;
}

void Day::ensure_julian()
{
    if (julian_valid) return;

    julian = to_julian(year, month, date);
    julian_valid = true;
}

void Day::ensure_ymd()
{
    if (ymd_valid) return;

    int ymd[3];
    from_julian(julian, ymd);

    year  = ymd[0];
    month = ymd[1];
    date  = ymd[2];
    ymd_valid = true;
}

int Day::to_julian(const int year, const int month, const int date)
{
    int jy = year;
    if (year < 0) jy++;
    int jm = month;

    if (month > 2) jm++;
    else
    {
        jy--;
        jm += 13;
    }

    int jul = static_cast<int>(floor(365.25*jy) + floor(30.6001*jm))
                    + date + 1720995;

    // Gregorian calendar adopted Oct. 15, 1582.
    const int IGREG = 15 + 31*(10 + 12*1582);

    // Change over to Gregorian calendar.
    if (date + 31*(month + 12*year) >= IGREG)
    {
        int ja = (int) (0.01*jy);
        jul += 2 - ja + static_cast<int>(0.25*ja);
    }

    return jul;
}

void Day::from_julian(const int j, int ymd[])
{
    int ja = j;
   
    // The Julian day number of the adoption of the Gregorian calendar.
    const int JGREG = 2299161;

    // Cross-over to Gregorian calendar produces this correction.
    if (j >= JGREG)
    {
        int jalpha = static_cast<int>(
                (static_cast<float>(j - 1867216) - 0.25)/36524.25 );
        ja += 1 + jalpha - static_cast<int>(0.25*jalpha);
    }

    int jb = ja + 1524;
    int jc = static_cast<int>(
            6680.0 + (static_cast<float>(jb - 2439870) - 122.1)/365.25 );
    int jd = static_cast<int>(365*jc + (0.25*jc));
    int je = static_cast<int>((jb - jd)/30.6001);
    int date = jb - jd - static_cast<int>(30.6001*je);

    int month = je - 1;
    if (month > 12) month -= 12;

    int year = jc - 4715;
    if (month > 2) --year;
    if (year <= 0) --year;

    ymd[0] = year;
    ymd[1] = month;
    ymd[2] = date;
}
